﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml.Linq;

namespace Bing
{
    /// <summary>
    /// Contains an array of results returned for a request to the Phonebook <see cref="ApiHelper.SourceType"/>, as well as properties indicating total number and positioning of results.
    /// </summary>
    [DebuggerDisplay("Total={Total}, Results={Results.Count}, Errors={Errors.Count}")]
    public class PhonebookResponse: SearchResponseBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PhonebookResponse"/> class.
        /// </summary>
        public PhonebookResponse(XContainer element)
            : base(element)
        {
            Results = new List<PhonebookResult>();
        }

        /// <summary>
        /// Gets or sets a number that represents an <b><i>estimate</i></b> of the total number of results returned by the <see cref="PhonebookRequest"/>.
        /// </summary>
        /// <value>Returns a number that represents an <b><i>estimate</i></b> of the total number of results returned by the <see cref="PhonebookRequest"/>.</value>
        /// <remarks>
        /// Based on a number of technical reasons, which are beyond the scope of this documentation, the Bing Engine does not return an exact total for a <see cref="PhonebookRequest"/>.
        /// The Bing Web Service is designed to handle this approximation when paging through results. If the number of results remaining, in the actual total set of results from the specified offset, is equal to or greater than the number of results requested in the <see cref="PhonebookRequest"/>.Count field, the requested number of results is returned.
        /// If the number of results remaining is less than the number requested, the Bing Engine returns the remaining results without error.
        /// </remarks>
        public UInt32 Total { get; set; }

        /// <summary>
        /// Gets or sets the actual offset, from zero, for the starting point of the result set that is returned from the corresponding <see cref="PhonebookRequest"/>.
        /// </summary>
        /// <value>Returns the actual offset, from zero, for the starting point of the result set that is returned from the corresponding <see cref="PhonebookRequest"/>.</value>
        /// <remarks>Typically, <b>Offset</b> is used in conjunction with the <b>Count</b> field on the <see cref="PhonebookRequest"/> object, and the <b>Total</b> field on the <see cref="PhonebookResponse"/> object, to facilitate paging through long lists of results.</remarks>
        public UInt32 Offset { get; set; }

        /// <summary>
        /// Gets or sets the to the collection of results in the local search repository.
        /// </summary>
        /// <value>Link to the collection of results in the local search repository.</value>
        public string LocalSerpUrl { get; set; }

        /// <summary>
        /// Gets or sets a title that describes this set of Phonebook results.
        /// </summary>
        /// <value>Returns a title that describes this set of Phonebook results.</value>
        /// <remarks>For example: <b>Top local listings for pizza near Seattle, WA.</b></remarks>
        public string Title { get; set; }

        /// <summary>
        /// Gets or sets the List of results returned for this <see cref="PhonebookRequest"/>.
        /// </summary>
        /// <value>List of results returned for this <see cref="PhonebookRequest"/>.</value>
        public List<PhonebookResult> Results { get; set; }

        /// <summary>
        /// Parses the response from the Bing API.
        /// </summary>
        /// <param name="element">The XML from the Bing API</param>
        /// <returns>An <see cref="PhonebookResponse" /></returns>
        public static PhonebookResponse ParseElement(XContainer element)
        {
            PhonebookResponse phonebookResponse = new PhonebookResponse(element);
            if (phonebookResponse.Errors.Count > 0) return phonebookResponse;

            var phonebook = element.Element(XName.Get("Phonebook", ApiHelper.PhonebookNs));
            if (phonebook == null) return phonebookResponse;

            phonebookResponse.Total = ApiHelper.GetElementValueOrDefault<UInt32>(phonebook, "Total", ApiHelper.PhonebookNs);
            phonebookResponse.Offset = ApiHelper.GetElementValueOrDefault<UInt32>(phonebook, "Offset", ApiHelper.PhonebookNs);

            XElement phonebookResults = phonebook.Element(XName.Get("Results", ApiHelper.PhonebookNs));

            if (phonebookResults == null) return phonebookResponse;
            foreach (XElement result in phonebookResults.Elements(XName.Get("PhonebookResult", ApiHelper.PhonebookNs)))
            {
                PhonebookResult phonebookResult = new PhonebookResult();
                phonebookResult.Title = ApiHelper.GetElementValueOrDefault<string>(result, "Title", ApiHelper.PhonebookNs);
                phonebookResult.Url = ApiHelper.GetElementValueOrDefault<string>(result, "Url", ApiHelper.PhonebookNs);
                phonebookResult.BusinessUrl = ApiHelper.GetElementValueOrDefault<string>(result, "BusinessUrl", ApiHelper.PhonebookNs);
                phonebookResult.City = ApiHelper.GetElementValueOrDefault<string>(result, "City", ApiHelper.PhonebookNs);
                phonebookResult.CountryOrRegion = ApiHelper.GetElementValueOrDefault<string>(result, "CountryOrRegion", ApiHelper.PhonebookNs);
                phonebookResult.DisplayUrl = ApiHelper.GetElementValueOrDefault<string>(result, "DisplayUrl", ApiHelper.PhonebookNs);
                phonebookResult.Latitude = ApiHelper.GetElementValueOrDefault<double>(result, "Latitude", ApiHelper.PhonebookNs);
                phonebookResult.Longitude = ApiHelper.GetElementValueOrDefault<double>(result, "Longitude", ApiHelper.PhonebookNs);
                phonebookResult.PhoneNumber = ApiHelper.GetElementValueOrDefault<string>(result, "PhoneNumber", ApiHelper.PhonebookNs);
                phonebookResult.PostalCode = ApiHelper.GetElementValueOrDefault<string>(result, "PostalCode", ApiHelper.PhonebookNs);
                phonebookResult.ReviewCount = ApiHelper.GetElementValueOrDefault<UInt32>(result, "ReviewCount", ApiHelper.PhonebookNs);
                phonebookResult.StateOrProvince = ApiHelper.GetElementValueOrDefault<string>(result, "StateOrProvince", ApiHelper.PhonebookNs);
                phonebookResult.UniqueId = ApiHelper.GetElementValueOrDefault<string>(result, "UniqueId", ApiHelper.PhonebookNs);
                phonebookResult.UserRating = ApiHelper.GetElementValueOrDefault<double>(result, "UserRating", ApiHelper.PhonebookNs);
                phonebookResult.Business = ApiHelper.GetElementValueOrDefault<string>(result, "Business", ApiHelper.PhonebookNs);
                phonebookResult.Address = ApiHelper.GetElementValueOrDefault<string>(result, "Address", ApiHelper.PhonebookNs);

                phonebookResponse.Results.Add(phonebookResult);
            }

            return phonebookResponse;
        }


    }
}
