using System;
using System.Diagnostics;

namespace Bing
{
    /// <summary>
    /// Contains properties related to a result returned for a request to the Phonebook <see cref="ApiHelper.SourceType"/>.
    /// </summary>
    [DebuggerDisplay("Title: {Title}, {PhoneNumber} {DisplayUrl}")]
    public class PhonebookResult
    {
        /// <summary>
        /// Gets or sets the title of this result.
        /// </summary>
        /// <value>The title of this result.</value>
        public string Title { get; set; }

        /// <summary>
        /// Gets or sets the full URL for the result as a string.
        /// </summary>
        /// <value>Returns the full URL for the result as a string.</value>
        /// <remarks>
        /// The Url field contains the full URL to the page. This may contain extended information, such as instrumentation, and may be different than the DisplayUrl that you will want to display to the user. Typically, the Title is used to display the text of a hyperlink and the Url is used to specify the link target.<br />
        /// The URL returned is the same as the link target that accompanies the White Pages (residential) or Yellow Pages (commercial) listing for a Local Search.<br />
        /// </remarks>
        public string Url { get; set; }

        // TODO: Documentation Error: http://msdn.microsoft.com/en-us/library/dd250923.aspx, Incorrect data type, should be string
        /// <summary>
        /// Gets or sets the business URL.
        /// </summary>
        /// <value>If this result is a business with a web page, the URL is returned in this property.</value>
        public string BusinessUrl { get; set; }

        /// <summary>
        /// Gets or sets the name of the city of a result's address.
        /// </summary>
        /// <value>The name of the city of a result's address.</value>
        public string City { get; set; }

        // TODO: Documentation Error: http://msdn.microsoft.com/en-us/library/dd250877.aspx, Incorrect data type: Should be string
        /// <summary>
        /// Gets or sets the country or region name of a result.
        /// </summary>
        /// <value>The country or region name of a result.</value>
        public string CountryOrRegion { get; set; }

        // TODO: Documentation Error: http://msdn.microsoft.com/en-us/library/dd251027.aspx, Incorrect data type: Should be string
        /// <summary>
        /// Gets or sets the URL string to display to the user.
        /// </summary>
        /// <value>The URL string to display to the user.</value>
        public string DisplayUrl { get; set; }

        // TODO: Documentation Error: http://msdn.microsoft.com/en-us/library/dd251088.aspx, latitude is spelled incorrectly
        /// <summary>
        /// Gets or sets the latitude component of the location.
        /// </summary>
        /// <value>The latitude component of the location.</value>
        /// <remarks>
        /// Valid input values for Latitude range from -90 to 90. For accurate results, input values must be converted to the decimal representation of the Latitude value. Output values are returned in decimal format for results of SourceType.PhoneBook only.<br />
        /// Latitude lines are measured North and South from the equator in degrees, minutes, and seconds. The equator is defined as 0 degrees, the North Pole is 90 degrees north, and the South Pole is 90 degrees south. For example, the latitude of Seattle, Washington is 47 36' 23.17" N, which converts to the (positive) decimal value 47.603828.<br />
        /// </remarks>
        public double Latitude { get; set; }

        /// <summary>
        /// Gets or sets the longitude component of the location.
        /// </summary>
        /// <value>The longitude component of the location.</value>
        /// <remarks>
        /// Valid input values for Longitude range from -180 to 180. For accurate results, input values must be converted to the decimal representation of the Longitude value. Output values are returned in decimal format for results of SourceType.PhoneBook only.<br />
        /// Lines of longitude, also known as meridians, run between the North and South Poles. They measure east-west position in degrees, minutes, and seconds. The Prime Meridian, which runs through Greenwich, England, is assigned the value of 0 degrees. Meridians to the west of the Prime Meridian are assigned negative values, and those to the east are assigned positive values. For example, the longitude of Seattle, Washington is 122 19' 51.04" W, which converts to the (negative) decimal value of -122.328567.<br />
        /// </remarks>
        public double Longitude { get; set; }

        /// <summary>
        /// Gets or sets the phone number associated with the PhoneBook result for select markets.
        /// </summary>
        /// <value>The phone number associated with the PhoneBook result for select markets.</value>
        /// <remarks>
        /// The returned phone number is formatted in accordance with the formatting rules of the country or region in which the Location of the search exists. For example, if the location is in the United States, the phone number is formatted as (###) ###-####; if the location is in the United Kingdom, the phone number is formatted as #### ### ####.<br />
        /// The following markets are supported for the PhoneBook <see cref="ApiHelper.SourceType"/>:
        /// <table>
        /// <tr><td><b>Market Name</b></td><td><b>Language-Country/Region</b></td></tr>
        /// <tr><td>en-AU</td><td>English  Australia</td></tr>
        /// <tr><td>en-CA</td><td>English  Canada</td></tr>
        /// <tr><td>en-GB</td><td>English  United Kingdom</td></tr>
        /// <tr><td>en-US</td><td>English  United States</td></tr>
        /// <tr><td>es-US</td><td>Spanish  United States</td></tr>
        /// <tr><td>fr-CA</td><td>French  Canada</td></tr>
        /// <tr><td>fr-FR</td><td>French  France</td></tr>
        /// <tr><td>it-IT</td><td>Italian  Italy</td></tr>
        /// </table>
        /// </remarks>
        public string PhoneNumber { get; set; }

        // TODO: Documentation Error: Does not appear to be returned 
        /// <summary>
        /// Gets or sets the postal code.
        /// </summary>
        /// <value>The postal code.</value>
        public string PostalCode { get; set; }

        /// <summary>
        /// Gets or sets the number of reviews for this result.
        /// </summary>
        /// <value>The number of reviews for this result.</value>
        public UInt32 ReviewCount { get; set; }

        // TODO: Documentation Error: http://msdn.microsoft.com/en-us/library/dd250850.aspx , Incorrect Data Type.  Should be string
        /// <summary>
        /// Gets or sets the 2-character abbreviation for the state or province of a result.
        /// </summary>
        /// <value>The 2-character abbreviation for the state or province of a result.</value>
        public string StateOrProvince { get; set; }

        /// <summary>
        /// Gets or sets an identifier that uniquely identifies this result
        /// </summary>
        /// <value>An identifier that uniquely identifies this result</value>
        /// <remarks>You can input the value of PhonebookResult.UniqueID as the PhonebookRequest.LocId parameter in a request designed to lookup a specific phonebook entry.</remarks>
        public string UniqueId { get; set; }

        /// <summary>
        /// Gets or sets the average rating (1 is lowest, 10 is highest) given this result by Bing users.
        /// </summary>
        /// <value>The average rating (1 is lowest, 10 is highest) given this result by Bing users.</value>
        public double UserRating { get; set; }

        /// <summary>
        /// Gets or sets the name of the business for a result.
        /// </summary>
        /// <value>The name of the business for a result.</value>
        public string Business { get; set; }

        /// <summary>
        /// Gets or sets the street line of an address.
        /// </summary>
        /// <value>The street line of an address.</value>
        public string Address { get; set; }


    }
}