﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml.Linq;

namespace Bing
{
    /// <summary>
    /// Contains an array of results returned for a request to the Video <see cref="ApiHelper.SourceType"/>, as well as properties indicating total number and positioning of results.
    /// </summary>
    [DebuggerDisplay("Total={Total}, Results={Results.Count}, Errors={Errors.Count}")]
    public class VideoResponse: SearchResponseBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="VideoResponse"/> class.
        /// </summary>
        public VideoResponse(XContainer element):base (element)
        {
            Results = new List<VideoResult>();
        }

        /// <summary>
        /// Gets or sets a number that represents an <b><i>estimate</i></b> of the total number of results returned by the <see cref="VideoRequest"/>.
        /// </summary>
        /// <value>Returns a number that represents an <b><i>estimate</i></b> of the total number of results returned by the <see cref="VideoRequest"/>.</value>
        /// <remarks>
        /// Based on a number of technical reasons, which are beyond the scope of this documentation, the Bing Engine does not return an exact total for a <see cref="VideoRequest"/>.
        /// The Bing Web Service is designed to handle this approximation when paging through results. If the number of results remaining, in the actual total set of results from the specified offset, is equal to or greater than the number of results requested in the <see cref="VideoRequest"/>.Count field, the requested number of results is returned.
        /// If the number of results remaining is less than the number requested, the Bing Engine returns the remaining results without error.
        /// </remarks>
        public UInt32 Total { get; set; }

        /// <summary>
        /// Gets or sets the actual offset, from zero, for the starting point of the result set that is returned from the corresponding <see cref="VideoRequest"/>.
        /// </summary>
        /// <value>Returns the actual offset, from zero, for the starting point of the result set that is returned from the corresponding <see cref="VideoRequest"/>.</value>
        /// <remarks>Typically, <b>Offset</b> is used in conjunction with the <b>Count</b> field on the <see cref="VideoRequest"/> object, and the <b>Total</b> field on the <see cref="VideoResponse"/> object, to facilitate paging through long lists of results.</remarks>
        public UInt32 Offset { get; set; }

        /// <summary>
        /// Gets or sets the results.
        /// </summary>
        /// <value>The results.</value>
        public List<VideoResult> Results { get; set; }

        /// <summary>
        /// Parses the response from the Bing API.
        /// </summary>
        /// <param name="element">The XML from the Bing API</param>
        /// <returns>An <see cref="VideoResponse" /></returns>
        public static VideoResponse ParseElement(XContainer element)
        {
            VideoResponse videoResponse = new VideoResponse(element);
            if (videoResponse.Errors.Count > 0) return videoResponse;

            var video = element.Element(XName.Get("Video", ApiHelper.VideoNs));
            if (video == null) return videoResponse;

            videoResponse.Total = ApiHelper.GetElementValueOrDefault<UInt32>(video, "Total", ApiHelper.VideoNs);
            videoResponse.Offset = ApiHelper.GetElementValueOrDefault<UInt32>(video, "Offset", ApiHelper.VideoNs);

            XElement videoResults = video.Element(XName.Get("Results", ApiHelper.VideoNs));

            if (videoResults == null) return videoResponse;
            foreach (XElement result in videoResults.Elements(XName.Get("VideoResult", ApiHelper.VideoNs)))
            {
                VideoResult videoResult = new VideoResult();
                videoResult.Title = ApiHelper.GetElementValueOrDefault<string>(result, "Title", ApiHelper.VideoNs);
                videoResult.SourceTitle = ApiHelper.GetElementValueOrDefault<string>(result, "SourceTitle", ApiHelper.VideoNs);
                videoResult.RunTime = ApiHelper.GetElementValueOrDefault<int>(result, "RunTime", ApiHelper.VideoNs);
                videoResult.PlayUrl = ApiHelper.GetElementValueOrDefault<string>(result, "PlayUrl", ApiHelper.VideoNs);
                videoResult.ClickThroughPageUrl = ApiHelper.GetElementValueOrDefault<string>(result, "ClickThroughPageUrl", ApiHelper.VideoNs);

                XElement thumbnailElement = result.Element(XName.Get("StaticThumbnail", ApiHelper.VideoNs));
                if (thumbnailElement != null)
                {
                    Thumbnail thumbnail = new Thumbnail();
                    thumbnail.ContentType = ApiHelper.GetElementValueOrDefault<string>(thumbnailElement, "ContentType", ApiHelper.VideoNs);
                    thumbnail.Url = ApiHelper.GetElementValueOrDefault<string>(thumbnailElement, "Url", ApiHelper.VideoNs);
                    thumbnail.Width = ApiHelper.GetElementValueOrDefault<UInt32>(thumbnailElement, "Width", ApiHelper.VideoNs);
                    thumbnail.Height = ApiHelper.GetElementValueOrDefault<UInt32>(thumbnailElement, "Height", ApiHelper.VideoNs);
                    thumbnail.FileSize = ApiHelper.GetElementValueOrDefault<UInt32>(thumbnailElement, "FileSize", ApiHelper.VideoNs);
                    videoResult.StaticThumbnail = thumbnail;
                }
                videoResponse.Results.Add(videoResult);
            }

            return videoResponse;
        }
    }
}
